<?php
$mi__jsonData = file_get_contents('php://input');
if (!$mi__jsonData) {
    http_response_code(400); // Bad Request
    echo 'bad input data';
    exit;
}
// Decode the JSON data into a PHP associative array
$mitilena_data = json_decode($mi__jsonData, true);




// Check if decoding was successful
if ($mitilena_data !== null && isset($mitilena_data['statusObj'])) {
    $mitilena_data = $mitilena_data['statusObj'];

    if (!isset($mitilena_data['secretKey'])) {
        http_response_code(400); // Bad Request
        echo "Error: Api key of Mitilena (Mitilena Token) is not specified. We can't verify if the request is genuine. Cancellation of a transaction.";
        exit;
    }
    $mitilenaToken = $mitilena_data['secretKey'];

    require('../../../wp-load.php');
    $mi_pluginObj = new WC_Gateway_Mitilena();

    if (!$mi_pluginObj->mitilenaToken || (strlen($mi_pluginObj->mitilenaToken) < 4)) {
        // JSON decoding failed
        http_response_code(400); // Bad Request
        echo "The site owner did not initialize the Mitilena Token.";
        exit;
     }
    
    if ($mitilenaToken != $mi_pluginObj->mitilenaToken) {
        http_response_code(400); // Bad Request
        echo "Error: API token does not match. The website owner must specify the correct Api Mitilena token.";
        exit;
    }
 

    if (!isset($mitilena_data['yourPaymentId'])) {
        http_response_code(400); // Bad Request
        echo "Error: No payment id specified";
        exit;
    }
    $mi__paymentId = $mitilena_data['yourPaymentId'];

    if (strlen($mi__paymentId) < 1) {
        http_response_code(400); // Bad Request
        echo "Error: too short payment id.";
        exit;
    }


    function mi__pay_escapeString($var) {
        $var=stripslashes($var);
        $var=htmlentities($var);
        $var=strip_tags($var);
        return $var;
    }

    // yourPaymentId
    try {
        $mi__order = new WC_Order(mi__pay_escapeString($mi__paymentId));
    } catch (Throwable $e) {
        http_response_code(400); // Bad Request
        echo "Error: wrong payment id (order id). {yourPaymentId: \"\"}";
        exit;
    }

    if (!isset($mitilena_data['wasPaid'])) {
        http_response_code(400); // Bad Request
        echo "Error: No payment status specified";
        exit;
    }
    $mi__wasPaid = $mitilena_data['wasPaid'];


    if ($mi__wasPaid == true) {

        $blockchainHash = '';
        if (isset($mitilena_data['blockchainHash'])) {
           $blockchainHash = mi__pay_escapeString($mitilena_data['blockchainHash']);
           update_post_meta( $mi__order->get_id(), 'miBlockchainHash', $blockchainHash );
        }
        

            // set user choosen order status, usually it is processing (e.g. completed)
            $mi__order->update_status($mi_pluginObj->order_status, $blockchainHash);
            
            // Reduce stock levels
            try {
                $mi__order->wc_reduce_stock_levels();
            } catch (Throwable $t) {
                //  echo $t->getMessage();
                try {
                 @$mi__order->reduce_order_stock();  
                } catch (Throwable $tt) {
                 // 2 functions does not remove the warehouse stock, mb send a notification to the user?
                 // may be later
                //  echo $tt->getMessage();
                }            
            }
               // update status

    echo 'success';
    } else {
        echo 'unpaid status received';
    }
    // Perform further processing or respond to the request
 } else {

    // JSON decoding failed
    http_response_code(400); // Bad Request
    echo "Invalid JSON data";
 }

exit;
?>
