<?php
/*
Plugin Name: Mitilena Pay WooCommerce cryptocurrency payments.
Description: Start accepting cryptocurrency in your shop. Implementation in 5 clicks. Registration in 30 seconds. Possibility of complete anonymity. Crypto payments in 3 seconds. Support for conversion from 147 fiat currencies. Supports 5 cryptocurrencies.
Author: BLFA, Mitilena Team
Author URI: https://mitilena.com

Big thanks to: Lafif Astahdziq
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

/**
 * Mitilena Payment Gateway.
 *
 * Provides a Mitilena Payment Gateway.
 */
add_action('plugins_loaded', 'init_mitilena_gateway_class');
function init_mitilena_gateway_class() {

    class WC_Gateway_Mitilena extends WC_Payment_Gateway {

        public $domain;

        /**
         * Constructor for the gateway.
         */
        public function __construct() {

            $this->domain = 'mitilena_payment';

            $this->id                 = 'mitilena_payment';
            $this->icon               = apply_filters('woocommerce_mitilena_gateway_icon', '');
            $this->has_fields         = false;
            $this->method_title       = __( 'Fast Cryptocurrency Payment Mitilena', 'mitilena_payment' );
            $this->method_description = __( 'Allows you to accept payments in cryptocurrency  to your wallet in the Mitilena Pay. Supports conversion from 147 fiat currencies to USDT and others crypto.', 'mitilena_payment' );

             
             

            // Load the settings.
            $this->init_form_fields();
             
            $this->init_settings();

            // Define user set variables
            $this->title        = $this->get_option( 'title' );
            $this->description  = $this->get_option( 'description' );
            $this->instructions = $this->get_option( 'instructions', $this->description );
            $this->order_status = $this->get_option( 'order_status', 'completed' );
 

            $this->mitilenaToken = $this->get_option( 'mitilenaToken' );

            $this->available_cryptocurrencies = $this->get_option( 'available_cryptocurrencies' );
          

            // Actions
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
            add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'thankyou_page' ) );

            // Customer Emails
            add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions' ), 10, 3 );

        }

        /**
         * Initialise Gateway Settings Form Fields.
         */
        public function init_form_fields() {


            $request_body = [
                'region' => 'Europe'
            ];
            
            $request_headers = [
                'content-type'  => 'application/json'
            ];
      
            
            $environment_url = 'https://mitilena.com/api/listSupportedCrypto_miPay/woocommerce';
    
     $result = wp_remote_request($environment_url, array(
        'method' => 'GET',
        'headers' => $request_headers,
        'timeout'     => 60, // added
        'redirection' => 5,  // added
        'blocking'    => true, // added
        'httpversion' => '1.0',
        'sslverify' => false
    ));


    $arrWithOptionsObjects =  json_decode($result['body']);

    $optionsArr = array();
    if (is_array($arrWithOptionsObjects)) {
        foreach ($arrWithOptionsObjects as $oneObj) {
                if (is_object($oneObj) && isset($oneObj->woocommerceId) && isset($oneObj->woocommercedSelectText)) {
                    $optionsArr[$oneObj->symbol.';;;;'.$oneObj->protocol.';;;;'.$oneObj->blockchain.';;;;'.$oneObj->contract.';;;;'.$oneObj->woocommerceId.';;;;'.$oneObj->working] =  __( ''.$oneObj->woocommercedSelectText.'', 'mitilena_payment' )  ;
                    // in the format
                    // symbol;;;;protocol;;;;blockchain;;;;contract;;;;woocommerceId
                    // contract & protocol can be null, woocommerceId in fact is symbol-protocol, no protocol
                }
        }
    } 

            $this->form_fields = array(
                'enabled' => array(
                    'title'   => __( 'Enable/Disable', 'mitilena_payment' ),
                    'type'    => 'checkbox',
                    'label'   => __( 'Enable Mitilena Payment', 'mitilena_payment' ),
                    'default' => 'yes'
                ),
                'title' => array(
                    'title'       => __( 'Title','mitilena_payment' ),
                    'type'        => 'text',
                    'description' => __( 'This controls the title which the user sees during checkout.','mitilena_payment' ),
                    'default'     => __( 'Fast Cryptocurrency Payment Mitilena', 'mitilena_payment' ),
                    'desc_tip'    => true,
                ),
                'order_status' => array(
                    'title'       => __( 'Order Status', 'mitilena_payment' ),
                    'type'        => 'select',
                    'class'       => 'wc-enhanced-select',
                    'description' => __( 'Choose whether status you wish after checkout.', 'mitilena_payment' ),
                    'default'     => 'wc-completed',
                    'desc_tip'    => true,
                    'options'     => wc_get_order_statuses()
                ), 

                'mitilenaToken' => array(
                    'title'       => __( 'Mitilena API Token', 'mitilena_payment' ),
                    'type'        => 'textarea',
                    'description' => __( 'You get the token for free after registering on the website https://mitilena.com','mitilena_payment' ),
                    'default'     => __('Your Mitilena API token', 'mitilena_payment'),
                    'desc_tip'    => true,
                ),
                'description' => array(
                    'title'       => __( 'Description', 'mitilena_payment' ),
                    'type'        => 'textarea',
                    'description' => __( 'Payment method description that the customer will see on your checkout.', 'mitilena_payment' ),
                    'default'     => __('You will be redirected to the payment gateway website and receive instructions to pay in cryptocurrency. Payment is made in seconds.', 'mitilena_payment'),
                    'desc_tip'    => true,
                ),
                'instructions' => array(
                    'title'       => __( 'Instructions', 'mitilena_payment'),
                    'type'        => 'textarea',
                    'description' => __( 'Instructions that will be added to the thank you page and emails.', 'mitilena_payment' ),
                    'default'     => __('Always save the transaction blockchain hash.', 'mitilena_payment'),
                    'desc_tip'    => true,
                ),

                'available_cryptocurrencies'    => array(
                    'title'             => __( 'Available Cryptocurrencies','mitilena_payment' ),
                    'type'              => 'multiselect',
                    'class'             => 'wc-enhanced-select',
                    'css'               => 'width: 400px;',
                    'default'           => 'tether-trc20',
                    'options'           =>  $optionsArr,
                    'custom_attributes' => array(
                        'data-placeholder' => __( 'You need to select at least one cryptocurrency', 'mitilena_payment'  ),
                    ),

                    
                ),

            );
        }

        /**
         * Output for the order received page.
         */
        public function thankyou_page() {
            if ( $this->instructions ) {
                // echo wpautop( wptexturize( $this->instructions ) );
            }
        }

        /**
         * Add content to the WC emails.
         *
         * @access public
         * @param WC_Order $order
         * @param bool $sent_to_admin
         * @param bool $plain_text
         */
        public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
            $blockchainHash = 'n/a';

            try {
                $blockchainHash = $order->get_meta('miBlockchainHash');
                if (!$blockchainHash) {
                    $blockchainHash = 'n/a';
                }
            } catch(Exception $e) {
                // nothing to do, default value setted on top
            }
            
            if ( $this->instructions && ! $sent_to_admin && 'mitilena_payment' === $order->get_payment_method() && $order->has_status( 'processing' ) ) {
                echo '<p>'.$this->instructions.' '.$blockchainHash.'</p>' . PHP_EOL;
            }
        }

        public function payment_fields(){

            ?>
            <style>
 
 .mi__native_select {
   /* Reset Select */
   appearance: none;
   outline: 10px yellow;
   border: 0;
   box-shadow: none;
   /* Personalize */
   flex: 1;
   padding: 0 1em;
   color: #fff;
   background-color: #2c3e50;
   background-image: none;
   cursor: pointer;
 }
 /* Remove IE arrow */
 .mi__native_select::-ms-expand {
   display: none;
 }
 /* Custom Select wrapper */
 .mi__select {
   position: relative;
   display: flex;
   width: 20em;
   height: 3em;
   border-radius: .25em;
   overflow: hidden;
 }
 /* Arrow */
 .mi__select::after {
   content: '\25BC';
   position: absolute;
   top: 0;
   right: 0;
   padding: 1em;
   background-color: #34495e;
   transition: .25s all ease;
   pointer-events: none;
 }
 /* Transition */
 .mi__select:hover::after {
   color: #f39c12;
 }
 
 /* Other styles*/
 .mi__wrapper {
   color: #fff;
 }
 

 /* position,transform*/
 .mi__container {
 margin-top:5px;
 }
 label[for=payment_method_mitilena_payment]::after{
    content: "";
  display: inline-block !important;
  vertical-align: middle !important;
  width: 50px;
  height: 29px;
  background: url('https://mitilena.com/img/logo.png') 5px no-repeat;
    background-size: auto;
  background-size: contain;
 }
     </style>
            <?php
            if ( $description = $this->get_description() ) {
                echo wpautop( wptexturize( $description ) );
            }

 if ($this->available_cryptocurrencies && is_array($this->available_cryptocurrencies)) {
    ?>
       <div id="custom_input">
    <?php _e('Choose which cryptocurrency you will pay with:', 'mitilena_payment')?>

    <div class="mi__wrapper">
    <div class= "mi__container">
  <div class="mi__select">
    <select class="mi__native_select" name="mi__selected_crypto">

    <?php


    
foreach ($this->available_cryptocurrencies as $mi_key => $mi_value) {
        $detailsArr = explode(';;;;', $mi_value);
?>
      <option value="<?php echo esc_html($mi_value) ?>" <?php if (!(isset($detailsArr[5]) && $detailsArr[5] === 'yes')) {
      echo 'disabled=disabled';  
    }    ?>>
        <?php 
     if (is_array($detailsArr) && isset($detailsArr[0])) {
        echo esc_html($detailsArr[0]);
        if (isset($detailsArr[1]) && strlen($detailsArr[1]) > 1) {
            echo ' ('.trim(esc_html($detailsArr[1])).')';
        }
      } else {
        echo 'n/a';
      } 
      ?>
        </option>
         
<?php
}

?>
   </select>
  </div>
</div> 

</div>
</div>
<?php
}
        }

        /**
         * Process the payment and return the result.
         *
         * @param int $order_id
         * @return array
         */
        public function process_payment( $order_id ) {

    
            $order = wc_get_order( $order_id );

            $status = 'wc-' === substr( $this->order_status, 0, 3 ) ? substr( $this->order_status, 3 ) : $this->order_status;


            // Iterating through each WC_Order_Item_Product objects
            $orderids = '';
            $productCounter = 0;
            foreach ($order->get_items() as $item_key => $item ):

    ## Using WC_Order_Item methods ##

    // Item ID is directly accessible from the $item_key in the foreach loop or
    $item_id = $item->get_id();

    ## Using WC_Order_Item_Product methods ##

    $product      = $item->get_product(); // Get the WC_Product object

    $product_id   = $item->get_product_id(); // the Product id
    
    $variation_id = $item->get_variation_id(); // the Variation id

    $item_type    = $item->get_type(); // Type of the order item ("line_item")

    $item_name    = $item->get_name(); // Name of the product

    if ($productCounter == 0) {
        $orderids .= '['.$item_name.'] '.$product_id .';;;';
    } else {        
        $orderids .= $product_id . ';;;';
    }

    $productCounter++;
endforeach;

            
// some cuttings
$maximumLength = 100;
$orderids = (strlen($orderids) > $maximumLength) ? substr($orderids,0,$maximumLength).'...' : $orderids;


// get blockchain detail
$mi__symbol = $order->get_meta('mi__symbol');
// $mi__protocol = $order->get_meta('mi__protocol');
$mi__blockchain = $order->get_meta('mi__blockchain');
// $mi__contractAddress = $order->get_meta('mi__contractAddress');

/*
                'cryptoReceiveSymbol'=> $mi__symbol, // detele
                                'contractAddress' => $mi__contractAddress, // delete
                'cryptoProtocol' => $mi__protocol, // delete 


*/
            $request_body = [
                'userSecret' => urlencode($this->mitilenaToken),
                'userPaymentId'=> urlencode($order->id),
                'blockchain'=> urlencode($mi__blockchain), // values from blockchain_id
                'fiatCode' => urlencode($order->currency), 
                'customerEmail' => urlencode($order->get_billing_email()),
            
                'fiatAmount'=> urlencode($order->get_total()),
                'userProductId' => urlencode($orderids),
                'language'=> 'en', // default en, not neccessary
                    'redirectUrl_yours' =>  urlencode($order->get_checkout_order_received_url()),
                'webHookPostUrl_yours' =>  urlencode(plugin_dir_url( __FILE__ ) .'mitilena-post-handler.php')
               
            ];
            
            $request_headers = [
                'content-type'  => 'application/json',
                'mitilena-signature'       => $this->mitilenaToken,
            ];
            
            $args = [
                'method' => 'POST',
                'httpversion'   => '1.0',
                'timeout' => 90,
                'headers' => $request_headers,
                'body' => ''
            ];
            
            $environment_url = 'https://mitilena.com/api/generatePayment/';
     $result = wp_remote_post($environment_url, array(
        'method' => 'POST',
        'headers' => $request_headers,
        'timeout'     => 60, // added
        'redirection' => 5,  // added
        'blocking'    => true, // added
        'httpversion' => '1.0',
        'sslverify' => false,
        'body' => json_encode($request_body))
    );

    $fullObj =  json_decode($result['body']);
    
    if (isset($result['response']) && isset($result['response']['code']) && $result['response']['code'] != 200) {
        $mi__err_message = 'Unknown error';
    if (isset($result['body'])) {
        $mi__err_message = 'Error: '.$result['body'];
    }

    wc_add_notice( 'adsadsad' , 'error' );
    
    return array(
        'result'   => 'failure',
        'redirect' => WC()->cart->get_checkout_url()
    );   
    }

    if (!isset($fullObj->paymentPage)) {
        http_response_code(400); // Bad Request
        echo "Error: No payment id specified";
        exit;
    }
    $paymentPage = $fullObj->paymentPage;

    if (strlen($paymentPage) < 5) {
        http_response_code(400); // Bad Request
        echo "Error: payment page is too short.";
        exit;
    }

    function mi_pay_escapeString($var) {
        $var=stripslashes($var);
        $var=htmlentities($var);
        $var=strip_tags($var);
        return $var;
    }

      update_post_meta( $order->id, 'miPaymentPage', mi_pay_escapeString($paymentPage) );
 
            // set pending payment status
            $order->update_status( 'on-hold' );
          
            // Remove cart
            WC()->cart->empty_cart();
          
            if ($fullObj->paymentPage) {
  // Return thankyou redirect
  return array(
    'result'    => 'success',
    'redirect'  => $fullObj->paymentPage
);
            } else {
  // Return thankyou redirect
  return array(
    'result'    => 'success',
    'redirect'  => $this->get_return_url( $order )
);
            }

    
        }
    }
}

add_filter( 'woocommerce_payment_gateways', 'add_mitilena_gateway_class' );
function add_mitilena_gateway_class( $methods ) {
    $methods[] = 'WC_Gateway_Mitilena';
    return $methods;
}

add_action('woocommerce_checkout_process', 'process_mitilena_payment');
function process_mitilena_payment(){

    if($_POST['payment_method'] != 'mitilena_payment')
        return;


    if( !isset($_POST['mi__selected_crypto']) || empty($_POST['mi__selected_crypto']) )
        wc_add_notice( __( 'You have chosen to pay with cryptocurrency, but have not chosen the cryptocurrency itself. Please, choose.', 'mitilena_payment' ), 'error' );

}

/**
 * Update the order meta with field value
 */
add_action( 'woocommerce_checkout_update_order_meta', 'mitilena_payment_update_order_meta' );
function mitilena_payment_update_order_meta( $order_id ) {
    if($_POST['payment_method'] != 'mitilena_payment')
        return;

        function mi__pay_escapeString($var) {
            $var=stripslashes($var);
            $var=htmlentities($var);
            $var=strip_tags($var);
            return $var;
        }

        $mi__selected_cryptoValue = mi__pay_escapeString($_POST['mi__selected_crypto']);

        $mi__detailsArr = explode(';;;;', $mi__selected_cryptoValue);
        if (is_array($mi__detailsArr) && isset($mi__detailsArr[0]) && isset($mi__detailsArr[1]) && isset($mi__detailsArr[2]) && isset($mi__detailsArr[3])) {
            // first symbol
            // second protocol
            // third blockchain
            // 4 contract addr

            update_post_meta( $order_id, 'mi__symbol',$mi__detailsArr[0] );
            update_post_meta( $order_id, 'mi__protocol', $mi__detailsArr[1] );
            update_post_meta( $order_id, 'mi__blockchain', $mi__detailsArr[2] );
            update_post_meta( $order_id, 'mi__contractAddress', $mi__detailsArr[3] );

        } else if (is_array($mi__detailsArr) && isset($mi__detailsArr[0])) {
            update_post_meta( $order_id, 'mi__symbol',$mi__detailsArr[0] );
        }
}

/**
 * Customize order received page
 */

// On "Order received" page (add a message)
add_filter( 'woocommerce_thankyou_order_received_text', 'thankyou_applied_coupon_message', 10, 2 );
function thankyou_applied_coupon_message( $text, $order ) {

    // Only in thankyou "Order-received" page
    if( ! is_wc_endpoint_url( 'order-received' ) )
        return; // Exit

    global $wp;

    // Get the order ID
    $mi__order_id  = absint( $wp->query_vars['order-received'] );

    if ( empty($mi__order_id) || $mi__order_id == 0 )
        return; // Exit;

        function mi_pay_escapeString($var) {
            $var=stripslashes($var);
            $var=htmlentities($var);
            $var=strip_tags($var);
            return $var;
        }
    
        // yourPaymentId
        try {
            $mi__orderObj = new WC_Order(mi_pay_escapeString($mi__order_id));
        } catch (Throwable $e) {
            http_response_code(400); // Bad Request
            echo "Error: wrong payment id (order id).";
            exit;
        }
        $method = $mi__orderObj->get_payment_method();
        if($method == 'mitilena_payment' && $mi__orderObj->is_paid()) {
            $miPaymentPageHtml = $mi__orderObj->get_meta('miPaymentPage');
            if (!$miPaymentPageHtml) {
                $miPaymentPageHtml = '#';
            }
            return $text.'<br /><p>'.__('Your order has', 'mitilena_payment').' <a target="_blank" href="'.$miPaymentPageHtml.'">'.__('been paid', 'mitilena_payment').'</a>. '.__('Wait for further information.', 'mitilena_payment').'</p><br />';

        } else if($method == 'mitilena_payment' && $mi__orderObj->is_paid() === false) {
            $miPaymentPageHtml = $mi__orderObj->get_meta('miPaymentPage');
            if (!$miPaymentPageHtml) {
                $miPaymentPageHtml = 'no_data_contact_support';
            }
            return $text.'<br /><p>'.__('Order placed but not paid.','mitilena_payment').' <a href="'.$miPaymentPageHtml.'" target="_blank">'.__('Payment link here.','mitilena_payment').'</a></p><br />';
        } else {
            return $text;

        }
}


/**
 * Display field value on the order edit page
 */
add_action( 'woocommerce_admin_order_data_after_billing_address', 'mitilena_checkout_field_display_admin_order_meta', 10, 1 );
function mitilena_checkout_field_display_admin_order_meta($order){
    // 
    // $method = get_post_meta( $order->id, '_payment_method', true );
    $method = $order->get_payment_method();
    if($method != 'mitilena_payment') {
        return;
    }
  
    $blockchainHash = $order->get_meta('miBlockchainHash');
    if (!$blockchainHash) {
        $blockchainHash = 'n/a';
    }

    $mi__symbol = $order->get_meta('mi__symbol');
    
    if (!$mi__symbol) {
        $mi__symbol = 'n/a';
    }
    
    $mi__blockchain = $order->get_meta('mi__blockchain');
    if (!$mi__blockchain) {
        $mi__blockchain = 'n/a';
    }
    $blockchainHumanName = 'n/a';

    if ($mi__blockchain === 'tether-erc20') {
        $blockchainHumanName = 'USDT, ERC20, Ethereum';
    } else if ($mi__blockchain === 'tether-trc20') {
        $blockchainHumanName = 'USDT, TRC20, Tron';
    } else if ($mi__blockchain === 'tether-bep20') {
        $blockchainHumanName = 'USDT, BEP20, BSC';
    } else if ($mi__blockchain === 'mitilena-own') {
        $blockchainHumanName = 'VMT, Mitilena';
    } else if ($mi__blockchain === 'apfcoin') {
        $blockchainHumanName = 'APFC, ERC20';
    } 

    
    echo '<p><strong>'.__( 'Blockchain hash' ,'mitilena_payment').':</strong> ' . esc_html($blockchainHash) . '</p>';
    echo '<p><strong>'.__( 'Crypto symbol','mitilena_payment' ).':</strong> ' . esc_html($mi__symbol) . '</p>';
    echo '<p><strong>'.__( 'Blockchain' ,'mitilena_payment').':</strong> ' . $blockchainHumanName . '</p>';
}